#ifndef CUBE_
#define CUBE_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "VertexArrayObject.h"

/**
 * \brief A simple cube.
 *
 * A simple cube that uses GLUT to draw itself.
 */
class Cube: public RawObject
{
public:
	/**
	 * Constuctor: Brings the object into a consistent state and initializes its size.
	 *
	 * \param size						The size of the cube.
	 */
	Cube (double size);

	/**
	 * Destructor.
	 */
	~Cube ();

	/**
	 * Implementation of the Draw routine. Draws a cube using GLUT.
	 */
	void Draw ();

	/**
	 * Implementation of the GetBoundingRadius routine.
	 */
	double GetBoundingRadius ();

	/**
	 * Implementation of the GetBoundingBox routine.
	 */
	BoundingBox GetBoundingBox ();

private:
	/** Hide the default constructor. */
	Cube ();

	/** Size of the cube. */
	double size;
};

/**
 * \brief A simple cube.
 *
 * A cube that uses vertex arrays.
 */ 
class Cube2: public VertexArrayObject
{
public:
	/**
	 * Constuctor: Brings the object into a consistent state and sets the size of the cube.
	 *
	 * \param size					The size of the cube.					
	 */
	Cube2 (float size);

	/**
	 * Destructor.
	 */
	~Cube2 ();

	/**
	 * Constructs the vertex, index and normal arrays.
	 *
	 * \param size					The size of the cube.
	 */
	void Construct (float size);

private:
	/** Hide the default constructor. */
	Cube2 ();

	/** Size of the cube. */
	double size;

	/** Pointer to the vertex data. */
	float * v_data;

	/** Pointer to the index data. */
	GLuint * indices;
};

/**
 * \brief A textured cube.
 *
 * A simple cube that displays the same texture on each side. Uses a display list for acceleration.
 */
class TexturedCube: public RawObject
{
public:
	/**
	 * Constructor: Brings the object into a consistent state.
	 *
	 * \param size					The size of the cube.
	 * \param texture_id			OpenGL texture name identifying the texture that shall be used to draw the cube.
	 * \param allow_lighting		States wheather lighting shall be turned off during rendering or not.
	 */
	TexturedCube (float size, GLuint texture_id, bool allow_lighting);

	/**
	 * Destructor.
	 */
	~TexturedCube ();

	/**
	 * Implementation of the Draw routine.
	 */
	void Draw ();

	/**
	 * Implementation of the GetBoundingRadius routine.
	 */
	double GetBoundingRadius ();

	/**
	 * Implementation of the GetBoundingBox routine.
	 */
	BoundingBox GetBoundingBox ();


private:
	/** Hide the default constructor. */
	TexturedCube ();

	/**
	 * Constructs a display list for the cube.
	 */
	void ConstructDisplayList ();

	/** Size of the cube divided by two. */
	double half_size;

	/** OpenGL texture name identifying the texture that shall be used for drawing. */ 
	GLuint texture_id;

	/** OpenGL identifier that specifies the display list to be called for drawing. */
	GLuint list_id;

	/** Is lighting allowed for drawing? */
	bool allow_lighting;
};


#endif