#ifndef DEMO_
#define DEMO_

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "Scene.h"

#define DEMO_NOT_YET_STARTED		-1

#define DELTA_X				2.0f
#define DELTA_Y				2.0f
#define DELTA_Z				2.0f
#define DELTA_ROLL			1.0f
#define DELTA_PITCH			1.0f
#define DELTA_YAW			1.0f


/**
 * \brief A demo consisting of a sequence of scenes.
 *
 * This is the main class. When the demo is played, a sequence of scenes are rendered either in fullscreen mode
 * or in a window of an arbitrary size. The demo supports blending between scenes using the OpenGL accumulation
 * buffer. Apart from the ability to start and stop the playback of the demo it also provides a mechanism to 
 * monitor the scenes. When entering this monitoring mode, playback of the demo is stopped and the camera may
 * be moved around freely. Additionally, the bounding volumes that are currently used to apply view frustum 
 * culling are displayed as well as the camera that captures the scene and its view frustum. Furthermore, all
 * the objects are drawn with the level of detail that it used for the camera that captures the scene.
 *
 * The demo also contains some basic properties for illumination that are common to all scenes, such
 * as the minimal ambient light during playback and during monitoring mode or the properties of the
 * specular reflections.
 */
class Demo
{
public:
	/**
	 * Constructor: Brings the demo into a consistent state.
	 */
	Demo ();

	/**
	 * Destructor: Frees ressources needed by the demo
	 */ 
	~Demo ();

	/**
	 * Initializes the demo, its scenes and OpenGL states.
	 *
	 * \param argc					The number of arguments in argv
	 * \param argv					Array of strings specifiying the mode in which the demo gets played
	 *								If nothing is specified, the demo plays at 640x480 in windowed mode.
	 *								If there are two numbers, the demo plays at this resolution in windowed mode.
	 *								If there is one string, the demo tries to play in the fullscreen mode specified
	 *								by that string. This string must be of the format "WIDTHxHEIGHT:BPP@REFRESHRATE".
	 */
	void Init (int argc, char * argv[]);

	/**
	 * Starts the playback of the demo. 
	 */
	void Play ();

private:
	/**
	 * Switches monitoring mode on respectively off. 
	 */
	void SwitchMonitor ();

	/**
	 * Switches playback on respectively off.
	 */
	void SwitchPlayback ();

	/**
	 * Updates the elapsed demo time. The time in only increased if the demo is actually playing.
	 */ 
	void UpdateTime ();

	/**
	 * Prints a short description to the console.
	 */
	void PrintUsage ();

	/**
	 * The display loop of the demo. Handles the drawing of the scenes.
	 */
	void DisplayLoop ();

	/**
	 * Idle loop, does nothing.
	 */
	void IdleLoop ();

	/**
	 * Reshape function, called whenever the display window gets resized.
	 *
	 * \param w						The new width of the viewport
	 * \param h						The new height of the viewport
	 */
	void Reshape (int w, int h);

	/**
	 * Keyboard function, called whenever a regular keyboard key is pressed.
	 *
	 * \param key					The key being pressed.
	 */
	void Keyboard (unsigned char key, int x, int y);

	/**
	 * Special keyboard input function, called whenever a special keyboard key is pressed.
	 *
	 * \param key					The key being pressed
	 */
	void KeyboardSpecial (int key, int x, int y);

	/** States if the demo has been initialized or not */
	bool is_initialized;

	/** Screen width of the demo. */
	int window_width;

	/** Screen height of the demo. */
	int window_height;

	/** States if the demo is in fullscreen_mode. NOT YET SUPPORTED. */
	bool fullscreen_enabled;

	/** Describes the mode currently used for view frustum culling: C_NONE, C_BOUNDING_BOX, C_SPHERE or C_MIXED. */
	int culling_mode;		

	/** States if objects shall be drawn using different levels of detail. */
	bool level_of_detail_enabled;

	/** States if the demo is currently running in monitoring mode. */
	bool monitoring_mode;

	/** States if the demo is currently playing. */
	bool demo_playing;

	/** The camera used to monitor the scenes of the demo. */
	Camera * monitor_camera;

	/** Actual time of the demo in milliseconds */
	int demo_time;

	/** Last valid time measurement used to update the demo time. */
	int last_measure_time;

	/** Current frame rate. */
	int frame_rate;

	/** States when the frame rate was evaluated for the last time. */
	int last_frame_rate_time;

	/** Simple counter used for the frame rate calculation. */
	int incremental_rate;

	/** Buffer used for string creation. */
	char buffer[200];

//-----------------------------------------------------------------------------------------------------
//									GLUT WRAPPER
//-----------------------------------------------------------------------------------------------------
	/**
	 * This static function is required by glut.
	 */
	static void static_display ();

	/**
	 * This static function is required by glut.
	 */
	static void static_idle ();

	/**
	 * This static function is required by glut.
	 */
	static void static_reshape (int w, int h);

	/** 
	 * This static function is required by glut.
	 */
	static void static_keyboard (unsigned char key, int x, int y);

	/**
	 * This static function is required by glut.
	 */
	static void static_keyboard_special (int key, int x, int y);

	/**
	 * This static variable determines the currently active demo.
	 */
	static Demo * current_demo;

//-----------------------------------------------------------------------------------------------------
//									SCENE MANAGEMENT
//-----------------------------------------------------------------------------------------------------
public:
	/**
	 * Adds a scene to the demo. The duration of this scene as well as the time needed
	 * to blend in the scene can be specified manually.
	 *
	 * \param scene					The scene which shall be added to the demo. Scenes are played
	 *								in the same order as they were added.
	 * \param duration				Duration of the scene in milliseconds.
	 * \param blend_in				Amount of time needed to blend the scene in. The scene before
	 *								is blended out linearly over the same time span.
	 */
	void AddScene (Scene * scene, int duration, int blend_in);

private:
	/** Data structure to hold the scenes. */
	typedef struct SceneListElem
	{
		SceneListElem * next;		/**< Pointer to the next element. */
		Scene * scene;				/**< The actual scene to be played. */
		int start_time;				/**< The absolute start time of the scene. */
		int end_time;				/**< The absolute end time of the scene. */
		int blend;					/**< This absolute time determines the start of the blending. */
	}SceneList;

	/** List constaining all the scenes of the demo. */
	SceneList * first_scene;

	/** Pointer to the currently active scene during playback. */
	SceneListElem * current_scene;

//------------------------------------------------------------------------------------------------------
//									LIGHTING
//------------------------------------------------------------------------------------------------------
	
	/** Global ambient light when not in monitoring mode. */
	float global_ambient_playback [4];

	/** Global ambient light when in monitoring mode. */
	float global_ambient_monitor [4];

	/** Global specular reflection. */
	float global_specular_reflection [4];

	/** Global shininess value. */
	int global_shininess;

	/**
	 * Sets the global ambient illumination when not in monitoring mode.
	 *
	 * \param red					Red part of the ambient light.
	 * \param green					Green part of the ambient light.
	 * \param blue					Blue part of the ambient light.
	 * \param alpha					Alpha value of the ambient light.
	 */
	void SetGlobalAmbientPlayback (float red, float green, float blue, float alpha);

	/**
	 * Sets the global ambient illumination when monitor mode.
	 *
	 * \param red					Red part of the ambient light.
	 * \param green					Green part of the ambient light.
	 * \param blue					Blue part of the ambient light.
	 * \param alpha					Alpha value of the ambient light.
	 */
	void SetGlobalAmbientMonitor (float red, float green, float blue, float alpha);

	/**
	 * Sets the global specular reflection.
	 *
	 * \param red					Red part of the specular reflection.
	 * \param green					Green part of the specular reflection.
	 * \param blue					Blue part of the specular reflection.
	 * \param alpha					Alpha value of the specular reflection.
	 */
	void SetGlobalSpecularReflection (float red, float green, float blue, float alpha);

	/**
	 * Sets the shininess of all the specular reflections.
	 *
	 * \param shininess				Determines the shininess of the specular reflections.
	 */
	void SetGlobalShininess (int shininess);
};


#endif