#ifndef GEOMETRY_
#define GEOMETRY_

#include <math.h>

#define MY_PI					3.141592f

/** 
 * \brief An axis aligned bounding box used for view frustum culling
 *
 * AABB Axis aligned bounding box. 
 */
typedef struct AABoundingBox
{
	float xmin;				/**< Minimal x coordinate value. */
	float xmax;				/**< Maximal x coordinate value. */
	float ymin;				/**< Minimal y coordinate value. */
	float ymax;				/**< Maximal y coordinate value. */
	float zmin;				/**< Minimal z coordinate value. */
	float zmax;				/**< Maximal z coordinate value. */
}BoundingBox;

/** 
 * \brief A point or vector in 3D space.
 *
 * Point or Vector in 3D space. 
 */
typedef struct Point3D
{
	float x;				/**< X coordinate value. */
	float y;				/**< Y coordinate value. */
	float z;				/**< Z coordinate value. */
}Vector, Point;

/**
 * \brief The corners of a bounding box.
 *
 * Points holding the corners of a bounding box 
 */
typedef struct BoundingBoxCorners
{
	Point p[8];				/**< Corners of the box. */
}BoxCorners;

/** 
 * \brief A plane in 3D space.
 * 
 * Plane given by ax + by + cz + d = 0, where n = (a,b,c)'. 
 */
typedef struct Plane3D
{
	Vector n;				/**< Normal vector of the plane. */
	float d;				/**< Distance from (0, 0, 0). */
}Plane;

/**
 * \brief Some basic geometrical operations.
 *
 * This class cannot be instantiated. Its static routines, however, provide some basic geometrical operations.
 */
class Geometry
{
public:

	/** 
	 * Returns the intersection point of 3 planes or (0,0,0) if any two of them are parallel.
	 *
	 * \param p1						One of the three planes to be intersected with each other.
	 * \param p2						One of the three planes to be intersected with each other.
	 * \param p3						One of the three planes to be intersected with each other.
	 */
	static Point PlaneIntersection (const Plane& p1, const Plane& p2, const Plane& p3);

	/**
	 * Inverts a homogenous matrix. Matrices that only consist of a sequence of rotations and translations
	 * are always homogenous. This routine operates on double precision values.
	 *
	 * \param src						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. This matrix will be inverted.
	 * \param dest						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. The inverted matrix is written into this array.
	 */
	static void InvertHomogenousMatrix (double * src, double * dest);

	/**
	 * Inverts a homogenous matrix. Matrices that only consist of a sequence of rotations and translations
	 * are always homogenous. This routine operates on single precision values.
	 *
	 * \param src						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. This matrix will be inverted.
	 * \param dest						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. The inverted matrix is written into this array.
	 */
	static void InvertHomogenousMatrix (float * src, float * dest);

	/**
	 * Multipies two matrices. This routine operates on double precision values.
	 *
	 * \param m1						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. This matrix serves as the left operand of the multiplication.
	 * \param m2						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. This matrix serves as the right operand of the multiplication.
	 * \param dest						Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. The result of the multiplication is written into this array.
	 */
	static void MatrixMultiply (double * m1, double * m2, double * dest);

	/**
	 * Multipies a matrix with a vector.
	 *
	 * \param m							Pointer to an array of 16 consecutive values describing a 4x4 column 
	 *									major matrix. This matrix serves as the left operand of the multiplication.
	 * \param vect						A vector in 3D space serving as the right operand of the multiplication.
	 *									Its homogenous (4th) coordinate is supposed to be '1.0'.
	 * \param res						The result of the multiplication is written to this vector in 3D space.
	 */
	static void VectorMultiply (float * m, const Vector & vect, Vector & res);

	/**
	 * Normalizes a vector. If the vector is of length 0, it will stay that way.
	 *
	 * \param vect						The vector in 3D space to be normalized.
	 */
	static void VectorNormalize (Vector & vect);

	/**
	 * Returns the length of any given vector.
	 *
	 * \param vect						The vector of which the length shall be calculated.
	 */
	static float VectorLength (Vector & vect);

	/**
	 * Computes the enclosing angle between two vectors in radians. 
	 *
	 * \param v1						The first vector used for the calculation of the enclosing angle.
	 * \param v2						The second vector used for the calculation of the enclosing angle.
	 * \return							The enclosing angle of the two vectors in radians (0..PI).
	 */
	static float EnclosingAngle (Vector & v1, Vector & v2);

	/**
	 * Calculates the cross product of 2 vectors.
	 *
	 * \param v1						The first vector used for the calculation of the cross product.
	 * \param v2						The second vector used for the calculation of the cross product.
	 * \return							The vector resulting from the cross product of v1 and v2. 
	 */
	static Vector CrossProduct (const Vector& v1, const Vector& v2);

	/**
	 * Calculates the scalar product of 2 vectors.
	 *
	 * \param v1						The first vector used for the calculation of the scalar product.
	 * \param v2						The second vector used for the calculation of the scalar product.
	 * \return							The scalar product of the two vectors.
	 */
	static float ScalarProduct (const Vector &v1, const Vector &v2);

private:
	/**
	 * Hide the default constructor.
	 */
	Geometry ();
};

#endif