#ifndef GROUP_OBJECT_
#define GROUP_OBJECT_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "Camera.h"
#include "GraphicalObject.h"

/**
 * \brief A group of GraphicalObjects.
 *
 * A GroupObject is, as the name suggests, just a group of objects. A group object has no own appearance, it just
 * groups a set of objects together and provides them a common coordinate system.
 */
class GroupObject: public GraphicalObject
{
public:
	/** 
	 * Constructor: Brings the group object into a consistent state. No RawObject may be associated with a group object.
	 */
	GroupObject ();

	/** 
	 * Destuctor.
	 */
	~GroupObject ();

	/**
	 * Implementation of the Draw routine. Draws All the children.
	 *
	 * \param time					Time at which the group object shall be drawn.
	 * \param camera				Camera which captures the group object.
	 * \param monitor				Camera which monitors the whole scene, is equal to NULL if we are not in monitoring 
	 *								mode. 
	 * \param show_bounding_volume	States if the bounding volume used for the view frustum culling shall be drawn.
	 * \param culling_mode			Determines the mode used for the view frustum culling (C_NONE, C_BOUNDING_BOX, C_SPHERE
	 *								or C_MIXED).
	 * \param level_of_detail		States wheather level of detail is enabled or not.				
	 */
	void Draw (int time, Camera * camera, Camera * monitor, bool show_bounding_volume, int culling_mode, bool level_of_detail);

	/**
	 * Adds a child to the group of objects.
	 *
	 * \param child					Child to be added to this group.
	 */
	void AddChild (GraphicalObject * child);

	/**
	 * Prints information about the object to the console.
	 */
	void PrintToConsole ();

protected:
	/** 
	 * Implementation of the bounding sphere computation. Since the group itself does not occupy any space,
	 * the bounding radius is equal to zero. To take the volumina of the children into account,
	 * 'UpdateBoundingRadius ()' has to be called.
	 */
	void ComputeBoundingRadius ();

	/** 
	 * Implementation of the bounding box computation. Since the group itself does not occupy any space,
	 * the bounding box is equal to zero. To take the boxes of the children into account, 'UpdateBoundingBox ()'
	 * has to be called.
	 */
	void ComputeBoundingBox ();
private:

};

#endif
