#ifndef LIGHT_
#define LIGHT_

#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "PositionPath.h"

/**
 * \brief Can be used to illuminate a scene.
 *
 * The Light class just bundles together the code needed to apply lighting in OpenGL. Like any other object
 * that can be added to a scene, a Light has a position that may vary over time. Furthermore, it contains
 * values for the ambient, diffuse and specular illumination of a scene.
 */
class Light
{
public:
	/**
	 * Constructor: Initialized the light to default values.
	 */
	Light ();

	/**
	 * Destructor.
	 */
	~Light ();

	/**
	 * Applies the light to the current OpenGL environment.
	 *
	 * \param time					Specifies the time for which the position of the light shall be evaluated.
	 */
	void Apply (int time);

	/**
	 * Path of the light over time.
	 */
	PositionPath * position;

	/**
	 * Sets the ambient contribution of the light.
	 *
	 * \param red					Red part of the ambient illumination.
	 * \param green					Green part of the ambient illumination.
	 * \param blue					Blue part of the ambient illumination.
	 * \param alpha					Alpha value of the ambient illumination.
	 */
	void SetAmbient (float red, float green, float blue, float alpha);

	/**
	 * Set the diffuse contribution of the light.
	 *
	 * \param red					Red part of the diffuse illumination.
	 * \param green					Green part of the diffuse illumination.
	 * \param blue					Blue part of the diffuse illumination.
	 * \param alpha					Alpha value of the diffuse illumination.
	 */
	void SetDiffuse (float red, float green, float blue, float alpha);

	/**
	 * Sets the specular contributino of the light.
	 *
	 * \param red					Red part of the specular illuminaion.
	 * \param green					Green part of the specular illumination.
	 * \param blue					Blue part of the specular illumination.
	 * \param alpha					Alpha value of the specular illumination.
	 */
	void SetSpecular (float red, float green, float blue, float alpha);

	/**
	 * Binds the light to OpenGL
	 *
	 * \param id					Identifies this light under OpenGL.
	 */
	void Bind (int id);

	/**
	 * Returns a new unique value which identifies  the light under OpenGL.
	 */
	static int GetId ();

private:
	/** Ambient contribution. */
	float ambient[4];

	/** Diffuse contribution. */
	float diffuse[4];

	/** Specular contribution. */
	float specular[4];

	/** Stores the identifier which is used to bind the light to OpenGL */
	int id;

	/** holds id that will be distributed by the next call to 'GetId ()'. */
	static int current_id;

};

#endif