#ifndef SCENE_
#define SCENE_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "GraphicalObject.h"
#include "Light.h"

/**
 * \brief An element of the demo.
 *
 * A Scene is a part of a demo. It consists of one camera used to capture the scene, a set of lights
 * used to illuminate the scene and a set of objects which make up the environment of the scene.
 * A Scene is an abstract class, which means that every descendant that implements a scene has to
 * implement the way the scene is initialized (built). This initialization routine will be called when 
 * the demo which the scene belongs to gets initialized.
 */
class Scene
{
public:
	/**
	 * Constructor: Brings the object into a consistent state.
	 */
	Scene ();

	/**
	 * Destructor.
	 */
	~Scene ();

	/**
	 * Abstract function to initialize the scene that has to be implemented by the descendants. This function 
	 * gets called by the demo to which the scene belongs to.
	 */
	virtual void Init () = NULL;

	/**
	 * Renders the scene at any given time.
	 *
	 * \param time						Time in milliseconds at which the scene shall be rendered.
	 * \param culling_mode				Determines the mode used for the view frustum culling (C_NONE,
	 *									C_BOUNDING_BOX, C_SPHERE or C_MIXED).
	 * \param level_of_detail			States wheather level of detail is enabled or not.
	 */
	void Render (int time, int culling_mode, bool level_of_detail);

	/**
	 * Monitors the scene at any given time using a monitorning camera.
	 *
	 * \param time						Time in milliseconds at which the scene shall be rendered.
	 * \param monitor					The camera from which the scene shall be monitored.
	 * \param culling_mode				Determines the mode used for the view frustum culling (C_NONE,
	 *									C_BOUNDING_BOX, C_SPHERE or C_MIXED).
	 * \param level_of_detail			States wheather level of detail is enabled or not.
	 */
	void Monitor (int time, Camera * monitor, int culling_mode, bool level_of_detail);

	/**
	 * Copies the position of the camera that belongs to the scene at a specific time to the camera object 
	 * passed as a pointer.
	 *
	 * \param time						Time at which the position of the camera shall be copied.
	 * \param destination				Camera to which the position shall be copied.
	 */
	void CopyPosition (int time, Camera * destination);

	/**
	 * Prints information about the scene to the console.
	 */
	void PrintToConsole ();

protected:
	/**
	 * Adds an object to the scene.
	 *
	 * \param obj						The object which shall be added to the scene.
	 */
	void AddObject (GraphicalObject * obj);

	/** The camera that captures the scene. */
	Camera * camera;

private:

	/** Data structure to hold the graphical objects. */
	typedef struct GraphicalObjectElem
	{
		GraphicalObjectElem * next;					/**< Pointer to the element holding the next object. */
		GraphicalObject * graphical_object;			/**< Pointer to the object that belongs to the scene. */
	}GraphicalObjectList;

	/** The list containing the graphical objects. */
	GraphicalObjectList * first_object;

	/** Camera representation during monitor mode. */
	GraphicalObject * camera_object;

//--------------------------------------------------------------------------------------------------
//											LIGHT MANAGEMENT
//--------------------------------------------------------------------------------------------------
public:

protected:
	/**
	 * Adds a light to the scene.
	 *
	 * \param light						The light which shall be added to the scene.
	 */
	void AddLight (Light * light);

private:
	/** Data structure to hold the lights of the scene. */
	typedef struct LightElem
	{
		LightElem * next;						/**< Pointer to the element holding the next light. */
		Light * light;							/**< Pointer to the light that belongs to the scene. */
	}LightList;

	/** The list containing the lights. */
	LightList * first_light;

	/**
	 * Sets the 'current_light_id' to the next available OpenGL light identifier, provided that there is 
	 * one available.
	 */
	void NextLightId ();

	/**
	 * Current OpenGL light identifier.
	 */
	int current_light_id;

	/**
	 * Disables all lights that are not in used by this scene.
	 */
	void TurnOffLights ();

	/**
	 * Applies all the lights of the scene at any given time.
	 *
	 * \param time						Time in milliseconds at which the lights shall be applied.
	 */
	void ApplyLights (int time);
};

#endif
